﻿//+------------------------------------------------------------------+
//|                                               eBinaryTriumph.mq4 |
//|                                        Copyright 2018, komposter |
//|                                         http://www.komposter.me/ |
//+------------------------------------------------------------------+
#property copyright 		"Copyright 2018, komposter"
#property link      		"http://www.komposter.me/"
#property version			"1.01"
#property description	""
#property strict

/*
2018.12.12:
	[+] Initial release
2018.12.13 (1.01):
	[+] MQL5 ready
	[*] fibo from the open of the engulfing to the close of the confirmation candle
	[+] MarketEnter parameter
	[+] Enter by limit order
*/

input string 		Expert_Properties 			= "";				// Main properties:
input int 			Expert_Id 						= 3341;			// * MagicNumber

input string 		Strategy_Properties 			= "";				// Strategy properties:
input double		Lot								= 0.1;			// * Lot
input bool			MarketEnter						= true;			// * Enter by Market
input double		Limit_Lvl						= 50.0;			// * Limit level (%) (0 - disable enter by limit)
input double		StopLoss_Lvl					= 0.0;			// * StopLoss level (%)
input double		TakeProfit_Lvl					= 200.0;			// * TakeProfit level (%)

//---
datetime	last_bar = 0;

//---
#ifdef __MQL5__
	#include <MT4Orders.mqh> // https://www.mql5.com/en/code/16006
	#define Bid (::SymbolInfoDouble(_Symbol, ::SYMBOL_BID))
	#define Ask (::SymbolInfoDouble(_Symbol, ::SYMBOL_ASK))
#endif

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int OnInit()
{
	return(INIT_SUCCEEDED);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OnTick()
{
	if ( iTime( _Symbol, _Period, 0 ) <= last_bar ) return;
	last_bar = iTime( _Symbol, _Period, 0 );

	if ( IsExpertOrder() ) return;

	double level_0 = 0.0, level_100 = 0.0;
	if ( Signal_B( level_0, level_100 ) )
	{
		if ( MarketEnter ) 		OpenBuy 		 ( level_0 + (level_100-level_0)*StopLoss_Lvl/100.0,
															level_0 + (level_100-level_0)*TakeProfit_Lvl/100.0 );
		if ( Limit_Lvl != 0.0 ) OpenBuyLimit ( level_0 + (level_100-level_0)*Limit_Lvl/100.0,
															level_0 + (level_100-level_0)*StopLoss_Lvl/100.0,
															level_0 + (level_100-level_0)*TakeProfit_Lvl/100.0 );
	}
	if ( Signal_S( level_0, level_100 ) )
	{
		if ( MarketEnter ) 		OpenSell		 ( level_0 - (level_0-level_100)*StopLoss_Lvl/100.0,
															level_0 - (level_0-level_100)*TakeProfit_Lvl/100.0 );
		if ( Limit_Lvl != 0.0 ) OpenSellLimit( level_0 - (level_0-level_100)*Limit_Lvl/100.0,
															level_0 - (level_0-level_100)*StopLoss_Lvl/100.0,
															level_0 - (level_0-level_100)*TakeProfit_Lvl/100.0 );
	}
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool IsExpertOrder()
{
	for ( int z = OrdersTotal() - 1; z >= 0; z -- )
	{
		if ( !OrderSelect( z, SELECT_BY_POS, MODE_TRADES ) )
		{
			Print( "OrderSelect( ", z, ", SELECT_BY_POS, MODE_TRADES ) - Error #", GetLastError() );
			continue;
		}
		if ( OrderMagicNumber() != Expert_Id || OrderSymbol() != _Symbol ) continue;

		if ( OrderType() == OP_BUYLIMIT )
		{
			if ( H(1) >= OrderTakeProfit() )
			{
				Print( "Deleting buy-limit order #", OrderTicket(), " because it's TP had to be reached..." );
				if ( OrderDelete( OrderTicket() ) ) {}
			}
		}
		else if ( OrderType() == OP_SELLLIMIT )
		{
			if ( L(1) <= OrderTakeProfit() )
			{
				Print( "Deleting sell-limit order #", OrderTicket(), " because it's TP had to be reached..." );
				if ( OrderDelete( OrderTicket() ) ) {}
			}
		}

		return(true);
	}
	return(false);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool Signal_B( double &Level_0, double &Level_100 )
{
	if (  C(3) < O(3) && 
			C(2) > O(2) && 
			(C(2) - O(2)) > (O(2) - L(2)) && (C(2) - O(2)) > (H(2) - C(2)) &&
			C(2) >= H(3) && O(2) <= C(3) &&
			C(1) > O(1) &&
			C(1) > C(2)  )
	{
		Level_0		= O(2);
		Level_100	= C(1);

		if ( C(1) < Level_0 + (Level_100 - Level_0)*2.0 )
		{
			// engulfing candle is not allowed to engulf more than three candles before
			int engulfed = 0;
			for ( int i = 3; i < B()-1; i ++ )
			{
				if ( C(2) >= H(i) && O(2) <= C(i) )
				{
					engulfed ++;
					if ( engulfed > 3 ) return(false);
				}
				else
				{
					break;
				}
			}

			return(true);
		}
	}
	return(false);
}

bool Signal_S( double &Level_0, double &Level_100 )
{
			// 3rd candle was bullish
	if (  C(3) > O(3) && 
			// 2nd - bearish
			C(2) < O(2) && 
			// on the engulfing candle, the body must be larger than the wicks
			(O(2) - C(2)) > (C(2) - L(2)) && (O(2) - C(2)) > (H(2) - O(2)) &&
			// 2nd engulfs the 3rd
			C(2) <= L(3) && O(2) >= C(3) &&
			// 1st (last one) is bearish too
			C(1) < O(1) &&
			// close of the 2nd bearish candle is lower than the close of the engulfing bearish candle
			C(1) < C(2)  )
	{
		Level_0		= O(2);
		Level_100	= C(1);

		// confirmation candle did not pass the 200% retracement of the engulfing candle
		if ( C(1) > Level_0 - (Level_0 - Level_100)*2.0 )
		{
			// engulfing candle is not allowed to engulf more than three candles before
			int engulfed = 0;
			for ( int i = 3; i < B()-1; i ++ )
			{
				if ( C(2) <= L(i) && O(2) >= C(i) )
				{
					engulfed ++;
					if ( engulfed > 3 ) return(false);
				}
				else
				{
					break;
				}
			}

			return(true);
		}
	}
	return(false);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double O( int b ) { return( iOpen ( _Symbol, _Period, b ) ); }
double H( int b ) { return( iHigh ( _Symbol, _Period, b ) ); }
double L( int b ) { return( iLow  ( _Symbol, _Period, b ) ); }
double C( int b ) { return( iClose( _Symbol, _Period, b ) ); }
double B() { return( iBars( _Symbol, _Period ) ); }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OpenBuy( double sl, double tp )
{
	double op = NormalizeDouble( Ask, _Digits );
	sl = NormalizeDouble( sl, _Digits );
	tp = NormalizeDouble( tp, _Digits );

	Print( "Trying to open BUY at ", DoubleToString( op, _Digits ), "!" );
	if ( OrderSend( _Symbol, OP_BUY, Lot, op, 0, sl, tp, "", Expert_Id ) < 0 )
		Alert( "OrderSend error #", GetLastError(), "!!!" );
}

void OpenSell( double sl, double tp )
{
	double op = NormalizeDouble( Bid, _Digits );
	sl = NormalizeDouble( sl, _Digits );
	tp = NormalizeDouble( tp, _Digits );

	Print( "Trying to open SELL at ", DoubleToString( op, _Digits ), "!" );
	if ( OrderSend( _Symbol, OP_SELL, Lot, op, 0, sl, tp, "", Expert_Id ) < 0 )
		Alert( "OrderSend error #", GetLastError(), "!!!" );
}

void OpenBuyLimit( double op, double sl, double tp )
{
	op = NormalizeDouble( op, _Digits );
	sl = NormalizeDouble( sl, _Digits );
	tp = NormalizeDouble( tp, _Digits );

	Print( "Trying to set BUY-LIMIT at ", DoubleToString( op, _Digits ), "!" );
	if ( OrderSend( _Symbol, OP_BUYLIMIT, Lot, op, 0, sl, tp, "", Expert_Id ) < 0 )
		Alert( "OrderSend error #", GetLastError(), "!!!" );
}

void OpenSellLimit( double op, double sl, double tp )
{
	op = NormalizeDouble( op, _Digits );
	sl = NormalizeDouble( sl, _Digits );
	tp = NormalizeDouble( tp, _Digits );

	Print( "Trying to set SELL-LIMIT at ", DoubleToString( op, _Digits ), "!" );
	if ( OrderSend( _Symbol, OP_SELLLIMIT, Lot, op, 0, sl, tp, "", Expert_Id ) < 0 )
		Alert( "OrderSend error #", GetLastError(), "!!!" );
}
